/*******************************************************************************
+
+  LEDA 7.0  
+
+
+  _file_panel.c
+
+
+  Copyright (c) 1995-2023
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


#include <LEDA/graphics/file_panel.h>
#include <LEDA/system/file.h>
#include <LEDA/core/string.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

LEDA_BEGIN_NAMESPACE

#include <LEDA/graphics/x_basic.h>

const int file_load   = 0; 
const int file_save   = 1; 
const int file_back   = 3; 
const int file_forward = 2; 
const int file_cancel = 4;

static double scroll_off = 0;
static window* win_ptr = 0;

static int window_width = 0;
static int window_height = 0;

static string selected_file;

#if defined(__win32__)
static char sep = '\\';
#else
static char sep = '/';
#endif


static bool file_panel_closed = false;

static void win_del_handler(window* wp) { 
  file_panel_closed = true;
/*
  int but = wp->get_button("Cancel");
  if (but != -1) wp->set_focus_button(but);
*/
}


static void redraw(window* wp)
{ window& W = *wp;

  if (!W.is_open()) return;

  W.set_text_font();

  file_panel* fp = (file_panel*)W.get_inf();

  int yskip = fp->yskip;

  if (W.width() != window_width || W.height() != window_height) 
  { window_width = W.width();
    window_height = W.height();
    fp->update();
    return;
   }

  W.start_buffering();
  //W.clear();
  W.draw_box(W.xmin(),W.ymin(),W.xmax(),W.ymax(),white);

  const list<string>& dir_list = fp->get_dir_list();
  const list<string>& file_list = fp->get_file_list();

  double pix = W.pix_to_real(1);

  double xleft = W.xmin();
  double xright = W.xmax() - W.pix_to_real(W.scrollbar_width() + 12);
  double ytop = W.ymax();
//double ybot = W.ymin();

  double tw = W.text_width("H");

  double y = ytop + pix;

  double x0 = xleft + tw;
  double x1 = x0 + 20*tw;
  double x2 = x1 + 6*tw;

  y += scroll_off;

  //double y1 = 0;
  double y2 = 0;
  double ycur = 0;

  string s;
  forall(s,dir_list)
  { 
    string s1 = "";

    color clr  = "#eeeeee";
    color lclr = "#d0d0d0";
    color tclr = blue;

    time_t tf = time_of_file(s);
    string time_str = time_string(tf);

    string s2 = time_str.head(-3).replace(" ","   ");

    string s3 = "Folder";

    if (s == "..") 
    { s2 = "";
    //s3 = "Parent";
      s3 = "";
    }
    else
      s += "/";

    double y1 = y - W.pix_to_real(yskip);
    W.draw_box(xleft,y,W.xmax(),y1,clr);
    W.draw_rectangle(xleft,y,W.xmax(),y1,lclr);

    //double yt = y - 0.20*W.pix_to_real(yskip);
    double yt = y - 0.15*W.pix_to_real(yskip);

    double x3 = xright - W.text_width(s3);
    W.draw_text(x0,yt,s,tclr);
    W.draw_text(x1,yt,s1,tclr);
    W.draw_text(x2,yt,s2,tclr);
    W.draw_text(x3,yt,s3,tclr);
    y = y1; 
   }

  y2 = y;

  string current_fn = fp->get_file_name();

  forall(s,file_list)
  { 
    time_t tf = time_of_file(s);
    size_t sz = size_of_file(s);
    string time_str = time_string(tf);

    color clr = "#fcfcfc";
    color lclr = grey1;
    color tclr = "#333333";

    if (s == current_fn) 
    { clr = "#ffffe0";
      ycur = y;
     }

    //if (s == selected_file) clr = ivory;

/*
    string suf = s.tail(3).to_upper();
*/
    string suf = "";

    string s1 = suf;

    string s2 = time_str.head(-3).replace(" ","   ");

    string s3 = string("%d   KB",(int)sz/1024);
    
    double y1 = y - W.pix_to_real(yskip);

    W.draw_box(xleft,y,W.xmax(),y1,clr);
    W.draw_rectangle(xleft,y,W.xmax(),y1,lclr);

    //double yt = y - 0.20*W.pix_to_real(yskip);

    double yt = y - (W.pix_to_real(yskip+2)-W.text_height("H"))/2;

    if (s == "No matching Files.") 
    { tclr = grey2;
      s1 = "";
      s2 = "";
      s3 = "";
     }

    double x3 = xright - W.text_width(s3);

    W.draw_text(x0,yt,s,tclr);
    W.draw_text(x1,yt,s1,tclr);
    W.draw_text(x2,yt,s2,tclr);
    W.draw_text(x3,yt,s3,tclr);

    y = y1; 
  }

//W.draw_hline(y1,grey2);
  W.draw_hline(y2,grey2);
  W.draw_hline(y,grey2);


  if (ycur != 0)
  { double y1 = ycur;
    double y2 = ycur - W.pix_to_real(yskip);
    //W.draw_rectangle(xleft-1,y1,W.xmax(),y2,"#555555");
    W.draw_rectangle(xleft-1,y1,W.xmax(),y2,"#333333");
   }
  

  W.flush_buffer();
  W.stop_buffering();

}



static void update_scroll_pos(int steps)
{ 
  file_panel* fp = (file_panel*)win_ptr->get_inf();

  int yskip = fp->yskip;

  double d = win_ptr->pix_to_real(yskip);


  const list<string>& dir_list = fp->get_dir_list();
  const list<string>& file_list = fp->get_file_list();
  int files = dir_list.length() + file_list.length();
  //int h = win_ptr->height();

  int h = win_ptr->real_to_pix(win_ptr->ymax() - win_ptr->ymin());
  int lines = int(h/yskip);

  if (files <= lines) return;

  double max_off = d*(files-lines);
  scroll_off += steps*d;

  if (scroll_off < 0) scroll_off = 0;
  if (scroll_off > max_off) scroll_off = max_off;

  //if (win_ptr->has_scrollbar()) 
  win_ptr->set_scrollbar_pos(scroll_off/max_off);
  redraw(win_ptr);
}


static void scroll_down(int i)
{ //cout << "scroll_down: " << i << endl;
  update_scroll_pos(+1);
}

static void scroll_up(int i)
{ //cout << "scroll_up: " << i << endl;
  update_scroll_pos(-1);
}

static void scroll_drag(int i)
{ //cout << "scroll_drag: " << i << endl;
  if (i == -1 ) {
    // drag start
    return;
  }

  if (i == -2 ) {
    // drag end
    return;
  }

  file_panel* fp = (file_panel*)win_ptr->get_inf();

  //double pix = win_ptr->pix_to_real(1);

  int yskip = fp->yskip;

  const list<string>& dir_list = fp->get_dir_list();
  const list<string>& file_list = fp->get_file_list();

  int files = dir_list.length() + file_list.length();
  //int h = win_ptr->height();

  int h = win_ptr->real_to_pix(win_ptr->ymax() - win_ptr->ymin());
  int lines = int(h/yskip);


  if (lines >= files)
     scroll_off = 0;
  else
  { double f = double(i)/1000;
    //scroll_off = f*(files-lines)*20*pix; 
    scroll_off = f*(files-lines)*win_ptr->pix_to_real(yskip); 
   }

  redraw(win_ptr);
}
  

 
file_panel::~file_panel()
{ W.close();
  if (wp) wp->redraw();
 }


void file_panel::change_filter(char* descr)
{ window* call_win = window::get_call_window();
  file_panel* fp = (file_panel*)call_win->get_inf();
  int pos = 0;
  string s;
  forall(s,fp->descr_list)
  { if (s == string(descr)) break;
    pos++;
   }

  if (pos < fp->pat_list.length()) 
     fp->pat_string = fp->pat_list[fp->pat_list.get_item(pos)];

  fp->update();
}

void file_panel::change_dir(char* dir)
{ 
  window* call_win = window::get_call_window();
  file_panel* fp = (file_panel*)call_win->get_inf();
  *fp->dir_name = dir;
  fp->update();
  fp->dir_stack.clear();
  call_win->disable_button(file_forward);
 }



void file_panel::init(window* wptr, string& fname, string& dname)
{ wp = wptr;

  load_handler = 0;
  save_handler = 0;
  cancel_handler= 0;

  win_ptr = &W;

  W.set_window_delete_handler(win_del_handler);
  W.set_redraw(redraw);

  W.set_fixed_font();

  load_ptr = 0;
  save_ptr = 0;
  cancel_ptr=0;
  
  if (dname == "") dname = ".";

  // if dname is prefix of fname remove it
  if (fname.index(fname) == 0) 
      fname = fname.replace(dname,"");
  if (fname[0] == '\\') fname = fname.tail(-1);

  file_name  = &fname;
  dir_name  = &dname;

  W.set_inf(this);
  load_string = "Load File from ";
  //load_string += dname;
  save_string = "Save File to ";
  //save_string += dname;
  panel_init = 0;

#if defined(__win32__)
  mswin = true;
  if (fname.empty()) fname = "untitled";
#else
  mswin = false;
#endif

  char* s;
  if ((s = getenv("HOME")) != 0)
  { home_dir = s;
    dir_list0.append(s);
   }
 
  list<string> drives = get_disk_drives();
  string drv;
  forall(drv,drives) dir_list0.append(drv + sep);

  dir_list0.sort();
}

  

file_panel::file_panel(string& fname, string& dname) : 
           W(window::screen_width()/4,window::screen_width()/5,"File Panel")
{ 
  init(0,fname,dname); 
}

file_panel::file_panel(string& fname, string& dname, int width, int height) : 
           W(width,height,"File Panel")
{ 
  init(0,fname,dname); 
}

file_panel::file_panel(window& win, string& fname, string& dname): 
           W(window::screen_width()/4,window::screen_width()/5,"File Panel")
{ 
  init(&win,fname,dname); 
}


void file_panel::set_load_object(const file_panel_handle_base& f)
{ load_ptr = &f; }

void file_panel::set_save_object(const file_panel_handle_base& f)
{ save_ptr = &f; }

void file_panel::set_cancel_object(const file_panel_handle_base& f)
{ cancel_ptr = &f; }



void file_panel::init_panel()
{
  if (panel_init) return;

  W.set_fixed_font();
  int tw = W.real_to_pix(W.text_width("H"));
  int th = W.real_to_pix(W.text_height("H"));

  yskip = int(1.7*th);


  if (descr_string.tail(1) != "|") descr_string += "|";
  if (pat_string.tail(1) != "|") pat_string += "|";

  descr_list.clear();
  pat_list.clear();

  int p1 = 0;
  int p2 = 0;
  while (p1 < descr_string.length() && p2 < pat_string.length())
  { int q1 = descr_string.index("|",p1);
    int q2 = pat_string.index("|",p2);
    string descr = descr_string(p1,q1-1);
    string pat =  pat_string(p2,q2-1).replace_all(";"," ");
    descr_list.append(descr);
    pat_list.append(pat);
    p1 = q1+1;
    p2 = q2+1;
  }

/*
  pat_list.append("*.*");
  descr_list.append("All Files (*.*)");
*/

  pat_string = pat_list.front();
  descr_string = descr_list.front();

  string title;
  if ((load_handler || load_ptr) && (save_handler || save_ptr))
     title = "File Panel";
  else
  { if (load_handler || load_ptr) title = load_string;
    if (save_handler || save_ptr) title = save_string;
   }

  W.set_frame_label(title);

  //W.set_auto_button_layout(false);

  dir_item  = W.string_item(" Folder",*dir_name,dir_list0,10,change_dir);

  W.set_item_label_width(dir_item,int(9.5*tw));
  W.set_item_width(dir_item,52*tw);

//W.set_item_height(dir_item,40);

  W.set_button_height(40);

  W.button(" < ",file_back);
  W.button(" > ",file_forward);

  if (window::display_type() == "xx") W.disable_item(dir_item);

  W.disable_button(file_back);

  W.disable_button(file_forward);


  W.make_menu_bar();

  file_item = W.string_item(" Selection",*file_name);
  W.set_item_label_width(file_item,int(10.0*tw));
  //W.set_item_width(file_item,26*tw);
  W.set_item_width(file_item,24*tw);

  if (pat_list.length() > 0)
    pat_item = W.string_item("",descr_string,descr_list,10,change_filter);
  else
    pat_item = W.string_item("",pat_string,change_filter);

  W.set_item_label_width(pat_item,int(3.3*tw));
  W.set_item_width(pat_item,24*tw);

/*
  W.text_item("");
  W.hspace(100);
*/

  // invisible buttons
  W.button(".x");
  W.button(".x");
  W.button(".x");

  if (load_handler || load_ptr) W.fbutton("Load",file_load);
  if (save_handler || save_ptr) W.fbutton("Save",file_save);
  W.button("Cancel",file_cancel);


/*
  panel_item it1 = W.get_button_item(but1);
  panel_item it2 = W.get_button_item(but2);
  W.set_item_xpos(it1,100);
  W.set_item_xpos(it2,200);
*/

  panel_init++;
}


void file_panel::update()
{
  if (is_directory(*dir_name))
  {
    set_directory(*dir_name);
    string dir = get_directory();


    if (window::display_type() == "xx" && dir == start_dir) 
      W.disable_button(file_back);
    else
      W.enable_button(file_back);

    if (dir.length() == 2 && dir.tail(1) == ":") dir += sep;

    if (dir_list0.search(dir) == nil) 
    { dir_list0.append(dir);
      dir_list0.sort();
     }

    *dir_name = dir;

    W.set_menu(dir_item,dir_list0,8);

    if (pat_string == "")
       file_list = get_files(*dir_name);
    else
    { file_list.clear();
      string pat;
      forall_words(pat,pat_string)
      { list<string> L = get_files(*dir_name,pat);
        //if (!L.empty()) file_list.conc(L);
        string s;
        forall(s,L) { 
          if (pat == "*.*" || s.tail(3) == pat.tail(3)) file_list.append(s);
        }
       }
     }


    // delete hidden files 
    list_item it;
    forall_items(it,file_list)
    { string s = file_list[it];
      if (s[0] == '.') file_list.del_item(it);
     }

/*
    // mark special files
    forall_items(it,file_list)
    { string s = file_list[it];
      string path = *dir_name + sep + s;
      // symbolic links
      if (is_link(path)) file_list[it] = s + "@";
     }
*/


     if (file_list.empty()) {
      file_list.append("No matching Files.");
     }

    file_list.sort();

    dir_list.clear();

    list<string> tmp_dir_list = get_directories(*dir_name);
    tmp_dir_list.sort();

    //dir_list.append("..");

    string d;
    forall(d,tmp_dir_list) {
       if (d[0] == '.') continue;
       dir_list.append(d);
    }

// scrollbar: seeems not to work with graphwin PS/LaTex export

   int len = dir_list.length() + file_list.length();

   double h = W.real_to_pix(W.ymax() - W.ymin());
   double sb_sz = h/(yskip*len);

   if (sb_sz < 1)
   { if (!W.has_scrollbar())
     { scroll_off = 0;
       W.open_scrollbar(scroll_up,scroll_down,scroll_drag,sb_sz,0);
      }
    }
   else
   { scroll_off = 0;
     W.close_scrollbar();
    }
  }
  else
  { 
    dir_list.clear();
    string s = dir_name->replace_all("\\","/");

    panel P("File Panel");
    P.text_item("");
    P.text_item(string("\\bf\\blue %s:~~no such directory.",~s));
    P.text_item("");
    P.fbutton("continue");
    P.display(W,window::center,window::center);
    P.read();

    file_list.clear();
  }

  if (load_handler || load_ptr) { 
    // load (use first matching file as selection)
/*
    if (!file_list.empty()) *file_name = file_list.front();
*/
   }
  else
  { //save
/*
    string s = *file_name;
    int p = s.length()-1;
    while (p >= 0 && s[p] != '.') p--;
    if (p >= 0) s = s(0,p-1);
    if (s != "") s += pat_string.del("*");
    *file_name = s;
*/
  }


  if (W.is_open()) 
  { W.redraw_panel();
    redraw(&W);
  }
}


void file_panel::x_open(int xpos, int ypos)
{ 
  start_dir = get_directory();

  dir_list0.append(start_dir);
  dir_list0.sort();

  init_panel();

  W.set_clear_on_resize(false);

/*
  if (wp)
     W.display(*wp,xpos,ypos);
  else
     W.display(xpos,ypos);
*/

  W.display(xpos,ypos);

  W.set_topmost();

  W.set_fixed_font();
  int tw = W.real_to_pix(W.text_width("H"));

  int wi = 72*tw;
  int he = 70*tw;

  int dx = (wi - W.width())/2; 
  int dy = (he - W.height())/2; 

  W.resize(W.xpos()-dx,W.ypos()-dy,wi,he);


  file_panel_closed = false;

  window_width = W.width();
  window_height = W.height();

  update();

  //double px = W.pix_to_real(1);

  //W.set_fixed_font();

  int but = -1;
  double x,y;
  int event;
  unsigned long t;

  do {
      event = W.read_event(but,x,y,t);

      if (file_panel_closed) {
         but = file_cancel;
         break;
      }

      if (event != button_press_event) continue;

      if (but == MOUSE_BUTTON(4)) { 
           scroll_up(0);
           continue;
      }

      if (but == MOUSE_BUTTON(5)) { 
           scroll_down(0);
           continue;
      }

      if (but == MOUSE_BUTTON(1))
      { //int d = int((W.ymax() + scroll_off - y)/(20*px));
        int d = int((W.ymax() + scroll_off - y)/W.pix_to_real(yskip));

        //selected_file = "";

        string fn;
     
        if (d < dir_list.length())
        { list_item it = dir_list.get_item(d);
          fn = dir_list[it];
         }
        else
        { int dd = d - dir_list.length();
          if (dd < file_list.length())
          { list_item it = file_list.get_item(dd);
            string s = file_list[it];
            if (s != "No matching Files.") fn = dir_list[it];
           }
         }

/*
        redraw(&W);
        while (W.read_event(but,x,y,t) != button_release_event);
*/

        if (d < dir_list.length())
        { *dir_name = fn;
          dir_stack.clear();
          W.disable_button(file_forward);
         }
        else
          *file_name = fn;

        //selected_file = "";

         update();

        continue;
      }

      if (but == file_back) 
      { string old_dir = *dir_name;
        *dir_name = "..";
        update();
        if (*dir_name != old_dir)
        { dir_stack.push(old_dir);
          W.enable_button(file_forward);
         }
        continue;
      }

      if (but == file_forward) 
      { if (!dir_stack.empty())
        { *dir_name = dir_stack.pop();
          if (dir_stack.empty()) W.disable_button(file_forward);
          update();
         }
        continue;
      }
  }
  while (but != file_load && but != file_save && but != file_cancel);


  string fn = *dir_name + sep + *file_name;

  set_directory(start_dir);


  W.close();
  if (wp) wp->redraw();


  switch (but) {
     case file_load: //update();
                     if (load_ptr) 
                        load_ptr->operator()(fn);
                     else 
                        load_handler(fn);
                     break;

     case file_save: //update();
                     if (save_ptr) 
                        save_ptr->operator()(fn);
                     else 
                        save_handler(fn);
                     break;
  }
}


void file_panel::x_open() { x_open(window::center,window::center); }



void file_panel::ms_open()
{ 
/*
  if (!wp) { LEDA_EXCEPTION(1, "file_panel::ms_open: no window"); return; }
*/

  int dw = wp ? wp->draw_win : 0;

  char dname[128];
  strcpy(dname,*dir_name);

  char fname[128];
  strcpy(fname,*file_name);

  string filter;

  if (descr_string.tail(1) != "|") descr_string += "|";
  if (pat_string.tail(1) != "|") pat_string += "|";

  int p1 = 0;
  int p2 = 0;
  while (p1 < descr_string.length() && p2 < pat_string.length())
  { int q1 = descr_string.index("|",p1);
    int q2 = pat_string.index("|",p2);
    filter += descr_string(p1,q1);
    filter += pat_string(p2,q2);
    p1 = q1+1;
    p2 = q2+1;
  }

  for(int i=0; i < filter.length(); i++)
    if (filter[i] == '|') filter[i] = '\0';


  string def_ext = pat_string.del("*.");

  if (load_handler || load_ptr)
  { 
    if (x_choose_file(dw,0,load_string,filter,def_ext,dname,fname)) 
    { *file_name = fname;
      *dir_name = dname;
      string fn = *dir_name + "\\" + *file_name;
      if (load_ptr) 
         load_ptr->operator()(fn);  
      else 
         load_handler(fn);
     }
    //else cout << "x_choose_file failed" << endl;
   }

  if (save_handler || save_ptr)
  { 
    if (x_choose_file(dw,1,save_string,filter,def_ext,dname,fname))
    { *file_name = fname;
      *dir_name = dname;
      string fn = *dir_name + "\\" + *file_name;
      if (save_ptr) 
         save_ptr->operator()(fn);    
      else 
         save_handler(fn);
     }
    //else cout << "x_choose_file failed" << endl;
   }

}



void file_panel::ms_open(int, int) { ms_open(); }


LEDA_END_NAMESPACE
